// ignore_for_file: non_constant_identifier_names, depend_on_referenced_packages

import 'package:calendar_date_picker2/calendar_date_picker2.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:intl/intl.dart';
import 'package:zoom_widget/zoom_widget.dart';

class AtilaUi extends StatelessWidget {
  const AtilaUi({super.key});

  @override
  Widget build(BuildContext context) {
    return Container();
  }

  ///  ========================================
  ///  FECHAS
  ///  ========================================

  /// Devuelve 'yyyy-MM-dd HH:mm:ss'
  static String fechas_GetCurrentDateTime() {
    final DateTime now = DateTime.now();
    final DateFormat formatter = DateFormat('yyyy-MM-dd HH:mm:ss');
    final String formatted = formatter.format(now);
    return formatted;
  }

  /// Devuelve 'dd/MM/yyyy HH:mm:ss'
  static String fechas_GetCurrentARGDateTime() {
    final DateTime now = DateTime.now();
    final DateFormat formatter = DateFormat('dd/MM/yyyy HH:mm:ss');
    final String formatted = formatter.format(now);
    return formatted;
  }

  /// Devuelve 'dd/MM/yyyy'
  static String fechas_GetCurrentARGDate() {
    final DateTime now = DateTime.now();
    final DateFormat formatter = DateFormat('dd/MM/yyyy');
    final String formatted = formatter.format(now);
    return formatted;
  }

  /// Convierte date a 'yyyy-MM-dd'
  static String fechas_ConvertDateToYYYYMMDD(DateTime date) {
    final DateFormat formatter = DateFormat('yyyy-MM-dd');
    final String formatted = formatter.format(date);
    return formatted;
  }

  /// Convierte 'dd/MM/yyyy' a 'yyyy-MM-dd'
  static String fechas_ConvertYYYYMMDDString(String date) {
    var inputFormat = DateFormat('dd/MM/yyyy');
    var date1 = inputFormat.parse(date);

    var outputFormat = DateFormat('yyyy-MM-dd');
    var formatted = outputFormat.format(date1);
    return formatted;
  }

  /// Convierte 'yyyy/mm/dd' a 'yyyy-MM-dd'
  static String fechas_ConvertirDDMMYYYYString(String date) {
    var inputFormat = DateFormat('yyyy/mm/dd');
    var date1 = inputFormat.parse(date);

    var outputFormat = DateFormat('yyyy-MM-dd');
    var formatted = outputFormat.format(date1);
    return formatted;
  }

  /// Convierte date a 'dd/MM/yyyy'
  static String fechas_ConvertirDDMMYYYY(DateTime date) {
    final DateFormat formatter = DateFormat('dd/MM/yyyy');
    final String formatted = formatter.format(date);
    return formatted;
  }

  /// Devuelve 'yyyy-MM-dd HH:mm:ss'
  static String fechas_SingleDateToString(
    CalendarDatePicker2Type datePickerType,
    List<DateTime?> values,
  ) {
    var valueText = (values.isNotEmpty ? values[0] : null)
        .toString()
        .replaceAll('00:00:00.000', '');

    if (datePickerType == CalendarDatePicker2Type.single) {
      if (values.isNotEmpty) {
        if (values.length == 1) {
          values.add(values[0]);
        }

        String startDate =
            '${values[0]!.day.toString().padLeft(2, '0')}/${values[0]!.month.toString().padLeft(2, '0')}/${values[0]!.year}';

        valueText = startDate;
      } else {
        return 'null';
      }
    }

    return valueText;
  }

  /// Convierte CalendarDatePicker2Type a  'dd/mm/yyyy'
  static String fechas_RangeDateToString(
    CalendarDatePicker2Type datePickerType,
    List<DateTime?> values,
  ) {
    var valueText = (values.isNotEmpty ? values[0] : null)
        .toString()
        .replaceAll('00:00:00.000', '');

    if (datePickerType == CalendarDatePicker2Type.range) {
      if (values.isNotEmpty) {
        if (values.length == 1) {
          values.add(values[0]);
        }

        String startDate =
            '${values[0]!.day.toString().padLeft(2, '0')}/${values[0]!.month.toString().padLeft(2, '0')}/${values[0]!.year}';
        String endDate = values[1] != null
            ? '${values[1]!.day.toString().padLeft(2, '0')}/${values[1]!.month.toString().padLeft(2, '0')}/${values[1]!.year}'
            : 'null';

        valueText = '$startDate - $endDate';
      } else {
        return 'null';
      }
    }

    return valueText;
  }

  /// Convierte CalendarDatePicker2Type a  'dd/mm/yyyy - dd/mm/yyyy'
  static String fechas_RangeDateToStringYYYYMMDD(
    CalendarDatePicker2Type datePickerType,
    List<DateTime?> values,
  ) {
    var valueText = (values.isNotEmpty ? values[0] : null)
        .toString()
        .replaceAll('00:00:00.000', '');

    if (datePickerType == CalendarDatePicker2Type.range) {
      if (values.isNotEmpty) {
        if (values.length == 1) {
          values.add(values[0]);
        }

        String startDate =
            '${values[0]!.year}-${values[0]!.month.toString().padLeft(2, '0')}-${values[0]!.day.toString().padLeft(2, '0')}';
        String endDate = values[1] != null
            ? '${values[1]!.year}-${values[1]!.month.toString().padLeft(2, '0')}-${values[1]!.day.toString().padLeft(2, '0')}'
            : 'null';

        valueText = '$startDate - $endDate';
      } else {
        return 'null';
      }
    }

    return valueText;
  }

  /// Abre calendario y devuelve List<DateTime?>
  static Future<List<DateTime?>?> fechas_BuildCalendar(
      {required BuildContext context,
      required List<DateTime?> initialValue}) async {
    var results = await showCalendarDatePicker2Dialog(
      context: context,
      config: CalendarDatePicker2WithActionButtonsConfig(
        cancelButton: TextButton(
          onPressed: () {
            Navigator.pop(context);
          },
          child: const Text(
            'CANCELAR',
            style: TextStyle(color: Colors.black54),
          ),
        ),
        okButtonTextStyle:
            const TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
        calendarType: CalendarDatePicker2Type.single,
        selectedDayHighlightColor: Theme.of(context).primaryColor,
        weekdayLabelTextStyle: const TextStyle(
          color: Colors.black87,
          fontWeight: FontWeight.bold,
        ),
        controlsTextStyle: const TextStyle(
          color: Colors.black,
          fontSize: 15,
          fontWeight: FontWeight.bold,
        ),
      ),
      dialogSize: const Size(325, 430),
      initialValue: initialValue,
    );

    return results;
  }

  /// Abre calendario y devuelve List<DateTime?>
  static Future<List<DateTime?>?> fechas_BuildCalendarRange(
      {required BuildContext context,
      required List<DateTime?> initialValue}) async {
    var results = await showCalendarDatePicker2Dialog(
      context: context,
      config: CalendarDatePicker2WithActionButtonsConfig(
        cancelButton: TextButton(
          onPressed: () {
            Navigator.pop(context);
          },
          child:
              const Text('CANCELAR', style: TextStyle(color: Colors.black54)),
        ),
        okButtonTextStyle:
            const TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
        calendarType: CalendarDatePicker2Type.range,
        selectedDayHighlightColor: Theme.of(context).primaryColor,
        weekdayLabelTextStyle: const TextStyle(
          color: Colors.black87,
          fontWeight: FontWeight.bold,
        ),
        controlsTextStyle: const TextStyle(
          color: Colors.black,
          fontSize: 15,
          fontWeight: FontWeight.bold,
        ),
      ),
      dialogSize: const Size(325, 430),
      initialValue: initialValue,
    );

    return results;
  }

  ///  ========================================
  ///  UTILES
  ///  ========================================

  static String utils_GetCurrentDateTime() {
    final DateTime now = DateTime.now();
    final DateFormat formatter = DateFormat('yyyyMMddHHmmss');
    final String formatted = formatter.format(now);
    return formatted;
  }

  static const anchoPantallMobile = 900;
  static const anchoPantallTablet = 700;

  static bool device_esWebCli() {
    if (defaultTargetPlatform == TargetPlatform.iOS ||
        defaultTargetPlatform == TargetPlatform.android) {
      return false;
    } else if (defaultTargetPlatform == TargetPlatform.linux ||
        defaultTargetPlatform == TargetPlatform.macOS ||
        defaultTargetPlatform == TargetPlatform.windows) {
      return false;
    }
    return true;
  }

  ///  ========================================
  ///  DEVICE
  ///  ========================================

  static bool device_esAndroid() {
    return (defaultTargetPlatform == TargetPlatform.android);
  }

  static bool device_usaTacking() {
    return (defaultTargetPlatform == TargetPlatform.iOS ||
        defaultTargetPlatform == TargetPlatform.android);
  }

  static bool device_esMobile(double width) {
    return width < anchoPantallMobile;
  }

  ///  ========================================
  ///  ALERTS
  ///  ========================================

  static void alerts_simple(
      BuildContext context, String title, String message) {
    Widget cancelButton = TextButton(
      child: Text('OK',
          style: TextStyle(
              fontWeight: FontWeight.w400,
              color: Theme.of(context).primaryColor)),
      onPressed: () {
        Navigator.pop(context);
      },
    );

    AlertDialog alert = AlertDialog(
      title: Text(title),
      content: Text(message),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(
          Radius.circular(20),
        ),
      ),
      actions: [
        cancelButton,
      ],
    );

    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }

  static void alerts_simpleConRedireccion(BuildContext context, String title,
      String message, String acceptText, Function acceptFunction) {
    Widget acceptButton = TextButton(
      child: Text(
        acceptText,
        style: TextStyle(
            fontWeight: FontWeight.w600, color: Theme.of(context).primaryColor),
      ),
      onPressed: () => acceptFunction(),
    );

    AlertDialog alert = AlertDialog(
      title: Text(title),
      content: Text(message),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(
          Radius.circular(20),
        ),
      ),
      actions: [
        acceptButton,
      ],
    );

    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }

  static void alerts_loadingSpinner(BuildContext context, String label) {
    showDialog(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        return Dialog(
          elevation: 10,
          shape: const RoundedRectangleBorder(
            borderRadius: BorderRadius.all(
              Radius.circular(20.0),
            ),
          ),
          child: Container(
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(20),
              color: Colors.white,
            ),
            padding: const EdgeInsets.symmetric(horizontal: 30, vertical: 40),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                CircularProgressIndicator(
                  strokeWidth: 8,
                  valueColor: AlwaysStoppedAnimation<Color>(
                    Theme.of(context).primaryColor,
                  ),
                ),
                const SizedBox(height: 20),
                Text(
                  label,
                  textAlign: TextAlign.center,
                  style: const TextStyle(
                      fontSize: 14, fontWeight: FontWeight.w400),
                ),
              ],
            ),
          ),
        );
      },
    );
  }

  static void alerts_snackBar(BuildContext context, String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        backgroundColor: Colors.grey[800],
        elevation: 20.0,
        margin: const EdgeInsets.symmetric(horizontal: 20, vertical: 50),
        padding: const EdgeInsets.only(left: 10, bottom: 10, top: 10),
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(10)),
        ),
        action: SnackBarAction(
          label: 'OK',
          textColor: Colors.white,
          onPressed: () {},
        ),
        behavior: SnackBarBehavior.floating,
        duration: const Duration(seconds: 4),
        content: Text(
          message,
          style: const TextStyle(
            fontWeight: FontWeight.w500,
            fontSize: 14.0,
            color: Colors.white,
          ),
        ),
      ),
    );
  }

  static void alerts_snackBarError(BuildContext context, String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        backgroundColor: Colors.red[900],
        elevation: 20.0,
        margin: const EdgeInsets.symmetric(horizontal: 20, vertical: 50),
        padding: const EdgeInsets.only(left: 10, bottom: 10, top: 10),
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(
            Radius.circular(10),
          ),
        ),
        action: SnackBarAction(
          label: 'OK',
          textColor: Colors.white, // or some operation you would like
          onPressed: () {
            // this block runs when label is pressed
          },
        ),
        behavior: SnackBarBehavior.floating,
        duration: const Duration(seconds: 4),
        content: Text(
          message,
          style: const TextStyle(
            fontWeight: FontWeight.w500,
            fontSize: 14.0,
            color: Colors.white,
          ),
        ),
      ),
    );
  }

  static void alerts_confirmar(BuildContext context, String title,
      String message, String acceptText, Function acceptFunction) {
    Widget cancelButton = TextButton(
      child: const Text(
        'CERRAR',
        style: TextStyle(
          fontWeight: FontWeight.w400,
          color: Colors.grey,
        ),
      ),
      onPressed: () {
        Navigator.pop(context);
      },
    );

    Widget acceptButton = TextButton(
      child: Text(
        acceptText,
        style: TextStyle(
            fontWeight: FontWeight.w600, color: Theme.of(context).primaryColor),
      ),
      onPressed: () => acceptFunction(),
    );

    AlertDialog alert = AlertDialog(
      title: Text(title),
      content: Text(message),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(
          Radius.circular(20),
        ),
      ),
      actions: [cancelButton, acceptButton],
    );

    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }

  ///  ========================================
  ///  BOTONES
  ///  ========================================

  static Widget btn_add({
    required BuildContext context,
    required Function()? onPressed,
    required String? label,
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        height: 48,
        margin: const EdgeInsets.only(left: 15, right: 15, top: 15, bottom: 15),
        padding: const EdgeInsets.only(left: 15, right: 15),
        decoration: BoxDecoration(
          color: Colors.green,
          borderRadius: BorderRadius.circular(5),
          border: Border.all(color: Colors.green, width: 1),
        ),
        child: Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Icon(Icons.add, color: Colors.white, size: 20),
              const SizedBox(width: 5),
              Text(
                label!,
                style: const TextStyle(color: Colors.white, fontSize: 18),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static Widget btn_back({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'VOLVER',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        height: 48,
        width: 150,
        margin: const EdgeInsets.only(left: 15, top: 15),
        decoration: BoxDecoration(
          color: Colors.black38,
          borderRadius: BorderRadius.circular(5),
          border: Border.all(color: Colors.black38, width: 1),
        ),
        child: Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Icon(Icons.arrow_back, color: Colors.white, size: 16),
              const SizedBox(width: 5),
              Text(
                label,
                style: const TextStyle(color: Colors.white),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static Widget btn_search_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return Container(
      decoration: BoxDecoration(
        color: Colors.blue.shade800,
        borderRadius: BorderRadius.circular(5),
      ),
      child: IconButton(
        icon: const Icon(Icons.search, color: Colors.white),
        onPressed: () {
          onPressed!();
        },
      ),
    );
  }

  static Widget btn_search_big({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'Buscar',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 12),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: Row(
          children: [
            const Icon(Icons.search, color: Colors.white),
            const SizedBox(width: 6),
            Text(
              label,
              style: const TextStyle(color: Colors.white, fontSize: 16),
            ),
          ],
        ),
      ),
    );
  }

  static Widget btn_process({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'PROCESAR',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        height: 48,
        width: 150,
        decoration: BoxDecoration(
          color: Colors.green,
          borderRadius: BorderRadius.circular(5),
          border: Border.all(color: Colors.black38, width: 1),
        ),
        child: Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Icon(Icons.check, color: Colors.white, size: 16),
              const SizedBox(width: 5),
              Text(
                label,
                style: const TextStyle(color: Colors.white),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static Widget btn_pdf_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 11),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: const Row(
          children: [
            Icon(
              Icons.add,
              color: Colors.white,
            ),
            Icon(
              Icons.file_copy,
              color: Colors.white,
              size: 20,
            ),
          ],
        ),
      ),
    );
  }

  static Widget btn_pdf_big({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'Adjuntar',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 11),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: Row(
          children: [
            const Icon(Icons.article_rounded, color: Colors.white),
            const SizedBox(width: 6),
            Text(
              label,
              style: const TextStyle(color: Colors.white, fontSize: 16),
            ),
          ],
        ),
      ),
    );
  }

  static Widget btn_delete_big({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'ELIMINAR',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        height: 48,
        width: 150,
        margin: const EdgeInsets.only(left: 15, top: 15),
        decoration: BoxDecoration(
          color: Colors.red,
          borderRadius: BorderRadius.circular(5),
          border: Border.all(
            color: Colors.black38,
            width: 1,
          ),
        ),
        child: Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Icon(Icons.delete, color: Colors.white, size: 16),
              const SizedBox(width: 5),
              Text(
                label,
                style: const TextStyle(color: Colors.white),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static Widget btn_delete_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return Container(
      margin: const EdgeInsets.only(left: 5, right: 5),
      decoration: BoxDecoration(
        color: Colors.red.shade800,
        borderRadius: BorderRadius.circular(5),
      ),
      child: IconButton(
        icon: const Icon(Icons.delete, color: Colors.white),
        onPressed: () {
          onPressed!();
        },
      ),
    );
  }

  static Widget btn_confirm({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'CONFIRMAR',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        height: 48,
        width: 150,
        margin: const EdgeInsets.only(left: 15, top: 15),
        decoration: BoxDecoration(
          color: Colors.green,
          borderRadius: BorderRadius.circular(5),
          border: Border.all(
            color: Colors.black38,
            width: 1,
          ),
        ),
        child: Center(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Icon(Icons.check, color: Colors.white, size: 16),
              const SizedBox(width: 5),
              Text(
                label,
                style: const TextStyle(color: Colors.white),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static Widget btn_camara_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 11),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: const Row(
          children: [
            Icon(Icons.add, color: Colors.white),
            Icon(Icons.camera_alt, color: Colors.white),
          ],
        ),
      ),
    );
  }

  static Widget btn_camara_big({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'Adjuntar',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 11),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: Row(
          children: [
            const Icon(Icons.photo_camera, color: Colors.white),
            const SizedBox(width: 6),
            Text(
              label,
              style: const TextStyle(color: Colors.white, fontSize: 16),
            ),
          ],
        ),
      ),
    );
  }

  static Widget btn_calendar_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return Container(
      margin: const EdgeInsets.only(top: 10, right: 15),
      decoration: BoxDecoration(
        color: Colors.blue.shade800,
        borderRadius: BorderRadius.circular(5),
      ),
      child: IconButton(
        icon: const Icon(Icons.calendar_month_outlined, color: Colors.white),
        onPressed: () {
          onPressed!();
        },
      ),
    );
  }

  static Widget btn_calendar_big({
    required BuildContext context,
    required Function()? onPressed,
    String label = 'Calendario',
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 11),
        decoration: BoxDecoration(
          color: Colors.blue.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: Row(
          children: [
            const Icon(Icons.calendar_month_outlined, color: Colors.white),
            const SizedBox(width: 6),
            Text(
              label,
              style: const TextStyle(color: Colors.white, fontSize: 16),
            ),
          ],
        ),
      ),
    );
  }

  static Widget btn_add_small({
    required BuildContext context,
    required Function()? onPressed,
  }) {
    return InkWell(
      onTap: () {
        onPressed!();
      },
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 11, vertical: 12),
        decoration: BoxDecoration(
          color: Colors.green.shade800,
          borderRadius: BorderRadius.circular(5),
        ),
        child: const Row(
          children: [Icon(Icons.add, color: Colors.white)],
        ),
      ),
    );
  }

  ///  ========================================
  ///  CARDS
  ///  ========================================

  static Widget cardMobile({
    required BuildContext context,
    required Widget child,
  }) {
    return Container(
      padding: const EdgeInsets.only(top: 15, bottom: 5),
      margin: const EdgeInsets.symmetric(horizontal: 10, vertical: 10),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: Colors.white,
        boxShadow: const [
          BoxShadow(
              color: Colors.grey, offset: Offset(0.0, 1.0), blurRadius: 6.0),
        ],
      ),
      child: child,
    );
  }

  static Widget cardMobileReporte({required Widget child}) {
    return Container(
      padding: const EdgeInsets.only(top: 8, bottom: 8, right: 10, left: 5),
      margin: const EdgeInsets.symmetric(horizontal: 8, vertical: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: Colors.white,
        boxShadow: const [
          BoxShadow(
              color: Colors.grey, offset: Offset(0.0, 1.0), blurRadius: 6.0),
        ],
      ),
      child: child,
    );
  }

  static Widget cardInfo({required Widget child}) {
    return Card(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Padding(
            padding: EdgeInsets.all(8.0),
            child: Icon(Icons.info, color: Colors.blue, size: 30),
          ),
          Expanded(child: child),
        ],
      ),
    );
  }

  static Widget cardFoto({
    required BuildContext context,
    required Widget child,
    required void Function()? onPressed,
    required void Function()? onDownload,
  }) {
    double width = MediaQuery.of(context).size.width;
    bool esMobile = device_esMobile(width);
    if (!esMobile) {
      width = width * 0.15;
    }

    return Stack(
      children: [
        InkWell(
          onTap: () {
            openImage(child: child, context: context, onDownload: onDownload);
          },
          child: Container(
            width: width,
            height: width,
            margin: const EdgeInsets.all(6),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(10.0),
              border: Border.all(width: 0.4, color: Colors.grey),
            ),
            child: ClipRRect(
              borderRadius: BorderRadius.circular(10.0),
              child: child,
            ),
          ),
        ),
        if (onPressed != null)
          Positioned(
              top: 2,
              right: 2,
              child: IconButton(
                onPressed: onPressed,
                icon: Container(
                  height: 34,
                  width: 34,
                  decoration: const BoxDecoration(
                      borderRadius: BorderRadius.all(Radius.circular(40)),
                      color: Colors.white),
                  child: Icon(
                    Icons.cancel,
                    color: Theme.of(context).primaryColor,
                    size: 30,
                  ),
                ),
              )),
      ],
    );
  }

  static Future<String?> openImage({
    required BuildContext context,
    required Widget child,
    required void Function()? onDownload,
    String labelClose = 'CERRAR',
    String labelOpen = 'ABRIR EN NUEVA PESTAÑA',
  }) async {
    var result = await showGeneralDialog(
        context: context,
        barrierDismissible: true,
        barrierLabel:
            MaterialLocalizations.of(context).modalBarrierDismissLabel,
        barrierColor: Colors.black45,
        transitionDuration: const Duration(milliseconds: 200),
        pageBuilder: (BuildContext buildContext, Animation animation,
            Animation secondaryAnimation) {
          return Material(
            child: SafeArea(
              child: Container(
                width: MediaQuery.of(context).size.width,
                height: MediaQuery.of(context).size.height,
                padding: const EdgeInsets.only(bottom: 20),
                color: Colors.black,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    Expanded(
                      child: Zoom(
                          initTotalZoomOut: true, child: Center(child: child)),
                    ),
                    Container(
                      padding: const EdgeInsets.only(bottom: 10),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceAround,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Row(
                            children: [
                              TextButton.icon(
                                icon: const Icon(Icons.close,
                                    color: Colors.white, size: 30),
                                onPressed: () {
                                  Navigator.pop(context);
                                },
                                label: Text(
                                  labelClose,
                                  style: const TextStyle(
                                      color: Colors.white, fontSize: 20),
                                ),
                              ),
                              TextButton.icon(
                                icon: const Icon(
                                  Icons.open_in_browser,
                                  color: Colors.white,
                                  size: 30,
                                ),
                                onPressed: () {
                                  onDownload!();
                                },
                                label: Text(
                                  labelOpen,
                                  style: const TextStyle(
                                      color: Colors.white, fontSize: 20),
                                ),
                              ),
                            ],
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          );
        });
    if (result == null || result == 'cancel') return null;

    return result.toString();
  }

  ///  ========================================
  ///  EMPTY
  ///  ========================================

  static Widget emptyAlert({
    required BuildContext context,
    required String label,
    required IconData? icon,
  }) {
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 10),
          margin: const EdgeInsets.all(10),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(10),
            color: Colors.grey.shade200,
            boxShadow: [
              BoxShadow(color: Colors.grey.shade200, spreadRadius: 1),
            ],
          ),
          child: Column(
            children: [
              Icon(icon),
              const SizedBox(height: 5),
              Center(child: Text(label, textAlign: TextAlign.center)),
            ],
          ),
        ),
      ],
    );
  }

  static Widget emptyList({
    required BuildContext context,
    required String label,
  }) {
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
          margin: const EdgeInsets.all(10),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(10),
            color: Colors.grey.shade200,
            boxShadow: [
              BoxShadow(
                color: Colors.grey.shade200,
                spreadRadius: 1,
              ),
            ],
          ),
          child: Column(
            children: [
              const Icon(FontAwesomeIcons.tableList),
              const SizedBox(height: 10),
              Center(
                child: Text(
                  label,
                  textAlign: TextAlign.center,
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }

  ///  ========================================
  ///  ERROR
  ///  ========================================

  static Widget errorList({
    required BuildContext context,
    required String label,
  }) {
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
          margin: const EdgeInsets.all(10),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(10),
            color: Colors.grey.shade200,
            boxShadow: [
              BoxShadow(color: Colors.grey.shade200, spreadRadius: 1),
            ],
          ),
          child: Column(
            children: [
              const Icon(FontAwesomeIcons.circleExclamation),
              const SizedBox(height: 10),
              Center(
                child: Text(label, textAlign: TextAlign.center),
              ),
            ],
          ),
        ),
      ],
    );
  }

  ///  ========================================
  ///  TEXT FILEDS
  ///  ========================================

  static Widget textArea({
    required BuildContext context,
    required TextEditingController? controller,
    required String label,
  }) {
    return Container(
      margin: const EdgeInsets.only(top: 10),
      child: TextField(
        minLines: 3,
        maxLines: 10,
        keyboardType: TextInputType.multiline,
        controller: controller,
        decoration: InputDecoration(
          floatingLabelBehavior: FloatingLabelBehavior.always,
          fillColor: Colors.white,
          border: const OutlineInputBorder(
            borderRadius: BorderRadius.all(Radius.circular(5.0)),
            borderSide: BorderSide(color: Colors.grey),
          ),
          label: Text(
            label.toUpperCase(),
            style: const TextStyle(
                fontWeight: FontWeight.bold, color: Colors.blue),
          ),
          contentPadding: const EdgeInsets.only(
              bottom: 10.0, left: 10.0, right: 10.0, top: 20.0),
        ),
      ),
    );
  }

  static Widget textoBox({
    required BuildContext context,
    required TextEditingController? controller,
    required String label,
    TextInputType? keyboardType = TextInputType.text,
    bool? readOnly = false,
    Widget? suffixIcon,
    void Function()? onTap,
    String? placeholder,
    Function(String)? onChanged,
  }) {
    return Container(
      margin: const EdgeInsets.only(top: 10),
      child: TextField(
        controller: controller,
        readOnly: readOnly!,
        onTap: onTap,
        keyboardType: keyboardType,
        onChanged: onChanged,
        decoration: InputDecoration(
          suffixIcon: suffixIcon,
          filled: true,
          floatingLabelBehavior: FloatingLabelBehavior.always,
          fillColor: Colors.white,
          border: const OutlineInputBorder(
            borderRadius: BorderRadius.all(Radius.circular(5.0)),
            borderSide: BorderSide(color: Colors.grey),
          ),
          hintText: placeholder,
          label: Text(
            label.toUpperCase(),
            style: const TextStyle(
                fontWeight: FontWeight.bold, color: Colors.blue),
          ),
          contentPadding: const EdgeInsets.only(
              bottom: 10.0, left: 10.0, right: 10.0, top: 10.0),
        ),
      ),
    );
  }
}
